CREATE OR REPLACE PROCEDURE  p_cens_alta(	
	v_tipus IN cens.tipus%TYPE,
        v_pais IN cens.idPais%TYPE,
        v_regio IN cens.idRegio%TYPE,
        v_localitat IN cens.idLocalitat%TYPE,
        v_idCreat OUT cens.idCens%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_cens_alta';
        var_par_entrada VARCHAR2(400);
	e_tipusCens EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_tipusCens, -02290);
	e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
        var_pais cens.idPais%TYPE;
        var_regio cens.idRegio%TYPE;
        var_localitat cens.idLocalitat%TYPE;
BEGIN
    -- validaci parametres entrada
    v_idCreat := 0;
    var_par_entrada := 'Tipus:'||v_tipus||' Pais:'||v_pais||' Regio:'||v_regio||' Localitat:'||v_localitat;
    CASE
      WHEN v_tipus='E' and v_pais is null THEN
	RSP := 'ERROR: Pel tipus de cens estatal, es obligatori especificar el pais.';
        RAISE e_parmEntrada;
      WHEN v_tipus='R' and (v_pais is null or v_regio is null) THEN
	RSP := 'ERROR: Pel tipus de cens regional, es obligatori especificar el pais i la regio.';
        RAISE e_parmEntrada;
      WHEN v_tipus='L' and (v_pais is null or v_regio is null or v_localitat is null) THEN
	RSP := 'ERROR: Pel tipus de cens local, es obligatori especificar el pais, la regio i la localitat.';
        RAISE e_parmEntrada;
      ELSE  
	RSP := 'OK';
        var_pais := v_pais;
        var_regio := v_regio;
        var_localitat := v_localitat;
        -- segons el tipus de cens, es posen a null les variables que no corresponguin
        CASE
          WHEN v_tipus='G' THEN
            var_pais := null;
            var_regio := null;
            var_localitat:=null;
          WHEN v_tipus='E' THEN
            var_regio:=null;
            var_localitat:=null;
          WHEN v_tipus='R' THEN
            var_localitat:=null;
          ELSE  
            RSP := 'OK';
        END CASE;    
    END CASE;
    -- Fi validaci parametres entrada
    -- Insert a taula cens
    INSERT INTO cens(tipus, idpais, idregio, idlocalitat) 
      VALUES ( v_tipus, var_pais, var_regio, var_localitat)
      RETURNING idcens INTO v_idCreat;
    -- inserci correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada,v_idCreat || ' - ' || RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- valors duplicats en una columna amb ndex nic
	WHEN DUP_VAL_ON_INDEX THEN
		ROLLBACK;
		RSP := 'ERROR: el cens de tipus: '||v_Tipus||' ja existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' ||RSP);
		COMMIT;
	-- valor de tipus de cens incorrecte
	WHEN e_tipusCens THEN
		ROLLBACK;
		RSP := 'ERROR: el tipus de cens no es correcte.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' ||RSP);
		COMMIT;
	-- no existencia de codis amb clau forana a la taula corresponent
	WHEN e_codiNoExistent THEN
		ROLLBACK;
                CASE
                  WHEN INSTR(UPPER(SQLERRM), 'FK1_CENSPAIS')>0 THEN
                    RSP := 'ERROR: El codi de pais no existeix.';
                  WHEN INSTR(UPPER(SQLERRM), 'FK2_CENSREGIO')>0 THEN
                    RSP := 'ERROR: El codi de regio no existeix.';
                  WHEN INSTR(UPPER(SQLERRM), 'FK3_CENSLOCALITAT')>0 THEN
                    RSP := 'ERROR: El codi de localitat no existeix.';
                  ELSE
                    RSP := 'ERROR: codi no existent.';
                END CASE;  
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' ||RSP);
		COMMIT;
	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada,v_idCreat || ' - ' || RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
		RSP := 'ERROR En la inserci del cens:'||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' ||RSP);
		COMMIT;
END p_cens_alta;
/

CREATE OR REPLACE PROCEDURE  p_cens_baixa(	
	v_idCens IN cens.idCens%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_cens_baixa';
        var_par_entrada VARCHAR2(400);
        e_parmEntrada EXCEPTION;
        var_tipus cens.tipus%TYPE;
BEGIN
    var_par_entrada := 'idCens:'||v_idCens;
    -- validaci parametres entrada
    IF v_idCens is null THEN
	RSP := 'ERROR: S''ha d''especificar el cens a eliminar';
        RAISE e_parmEntrada;
    END IF;
    SELECT tipus INTO var_tipus FROM cens WHERE idcens=v_idCens;
    IF var_tipus='G' THEN
	RSP := 'ERROR: No es pot eliminar el cens Global, sempre ha d''existir';
        RAISE e_parmEntrada;
    END IF;
    -- Fi validaci parametres entrada
    -- Eliminacio
    DELETE FROM cens WHERE idcens=v_idCens;
    -- eliminacio correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- no existeix el cens que es vol esborrar
	WHEN NO_DATA_FOUND THEN
		ROLLBACK;
		RSP := 'ERROR: el cens : '||v_idCens||' no existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
      	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
		RSP := 'ERROR En l''eliminaci del cens:'||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
END p_cens_baixa;
/

CREATE OR REPLACE PROCEDURE  p_cens_modificacio(	
        v_idcens IN cens.idCens%TYPE,
	v_tipus IN cens.tipus%TYPE,
        v_pais IN cens.idPais%TYPE,
        v_regio IN cens.idRegio%TYPE,
        v_localitat IN cens.idLocalitat%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_cens_modificacio';
        var_par_entrada VARCHAR2(400);
	e_tipusCens EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_tipusCens, -02290);
	e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
        var_pais cens.idPais%TYPE;
        var_regio cens.idRegio%TYPE;
        var_localitat cens.idLocalitat%TYPE;
        var_tipus cens.tipus%TYPE;
        var_numCiutadans NUMBER;
BEGIN
    var_par_entrada := 'IdCens:' || v_idCens || ' Tipus:'||v_tipus||' Pais:'||v_pais||' Regio:'||v_regio||' Localitat:'||v_localitat;

    -- validaci parametres entrada
    CASE
      WHEN v_idCens is null THEN
	RSP := 'ERROR: S''ha d''especificar l''identificador del cens a modificar.';
        RAISE e_parmEntrada;
      WHEN v_tipus='E' and v_pais is null THEN
	RSP := 'ERROR: Pel tipus de cens estatal, es obligatori especificar el pais.';
        RAISE e_parmEntrada;
      WHEN v_tipus='R' and (v_pais is null or v_regio is null) THEN
	RSP := 'ERROR: Pel tipus de cens regional, es obligatori especificar el pais i la regio.';
        RAISE e_parmEntrada;
      WHEN v_tipus='L' and (v_pais is null or v_regio is null or v_localitat is null) THEN
	RSP := 'ERROR: Pel tipus de cens local, es obligatori especificar el pais, la regio i la localitat.';
        RAISE e_parmEntrada;
      ELSE  
	RSP := 'OK';
        var_pais := v_pais;
        var_regio := v_regio;
        var_localitat := v_localitat;
        -- segons el tipus de cens, es posen a null les variables que no corresponguin
        CASE
          WHEN v_tipus='G' THEN
            var_pais := null;
            var_regio := null;
            var_localitat:=null;
          WHEN v_tipus='E' THEN
            var_regio:=null;
            var_localitat:=null;
          WHEN v_tipus='R' THEN
            var_localitat:=null;
          ELSE  
            RSP := 'OK';
        END CASE;    
    END CASE;

    SELECT tipus INTO var_tipus FROM cens WHERE idCens=v_idCens;
    -- el cens de tipus GLOBAL no es pot modificar, sempre ha d'existir
    IF var_tipus='G' THEN
	RSP := 'ERROR: El cens GLOBAL no es pot modificar.';
        RAISE e_parmEntrada;
    END IF;    
    
    -- un cens que tingui ciutadans assignats, no es pot modificar, podrien resultar dades no coherents amb els ciutadans
    
    SELECT count(*) INTO var_numCiutadans FROM padrocens WHERE idCens=v_idCens;
    IF var_numCiutadans>0 THEN
	RSP := 'ERROR: El cens no es pot modificar, hi ha ciutadans assignats.';
        RAISE e_parmEntrada;
    END IF;    

    -- Fi validaci parametres entrada
    -- Update a taula cens
    UPDATE cens SET 
      tipus = v_tipus,
      idpais =  var_pais,
      idregio = var_regio,
      idlocalitat = var_localitat
      WHERE idcens=v_idCens;
    -- modificacio correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- no existeix l'opci que es vol modificar
	WHEN NO_DATA_FOUND THEN
		ROLLBACK;
		RSP := 'ERROR: el cens: '||v_idCens||' no existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- valors duplicats en una columna amb ndex nic
	WHEN DUP_VAL_ON_INDEX THEN
		ROLLBACK;
		RSP := 'ERROR: el cens de tipus: '||v_Tipus||' ja existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- valor de tipus de cens incorrecte
	WHEN e_tipusCens THEN
		ROLLBACK;
		RSP := 'ERROR: el tipus de cens no es correcte.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- no existencia de codis amb clau forana a la taula corresponent
	WHEN e_codiNoExistent THEN
		ROLLBACK;
                CASE
                  WHEN INSTR(UPPER(SQLERRM), 'FK1_CENSPAIS')>0 THEN
                    RSP := 'ERROR: El codi de pais no existeix.';
                  WHEN INSTR(UPPER(SQLERRM), 'FK2_CENSREGIO')>0 THEN
                    RSP := 'ERROR: El codi de regio no existeix.';
                  WHEN INSTR(UPPER(SQLERRM), 'FK3_CENSLOCALITAT')>0 THEN
                    RSP := 'ERROR: El codi de localitat no existeix.';
                  ELSE
                    RSP := 'ERROR: codi no existent.';
                END CASE;  
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
		RSP := 'ERROR En la modificacio del cens:'||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
END p_cens_modificacio;
/
