CREATE OR REPLACE PROCEDURE  p_ciutada_alta(	
	v_ciutada IN ciutada.idCiutada%TYPE,
        v_pais IN ciutada.idPais%TYPE,
        v_regio IN ciutada.idRegio%TYPE,
        v_localitat IN ciutada.idLocalitat%TYPE,
        v_dataNaixement IN VARCHAR2,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_ciutada_alta';
        var_par_entrada VARCHAR2(400);
        e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
BEGIN
    -- validaci parametres entrada
    var_par_entrada := 'idCiutada:'||v_ciutada||' Pais:'||v_pais||' Regio:'||v_regio||' Localitat:'||v_localitat||' DataNaixement:'||v_dataNaixement;
    CASE
      WHEN v_ciutada is null THEN
	RSP := 'ERROR: Es obligatori especificar l''identificador del ciutada.';
        RAISE e_parmEntrada;
      WHEN v_pais is null or v_regio is null or v_localitat is null THEN
	RSP := 'ERROR: Es obligatori especificar el pais, regio i localitat del ciutada.';
        RAISE e_parmEntrada;
      WHEN v_dataNaixement is null THEN
	RSP := 'ERROR: Es obligatori especificar la data de naixement del ciutada.';
        RAISE e_parmEntrada;
      ELSE  
	RSP := 'OK';
    END CASE;
    -- Fi validaci parametres entrada
    -- Insert a taula ciutada
    INSERT INTO ciutada(idCiutada, idpais, idregio, idlocalitat, dataNaixement) VALUES ( v_ciutada, v_pais, v_regio, v_localitat, v_dataNaixement);
    -- inserci correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- valors duplicats en una columna amb ndex nic
	WHEN DUP_VAL_ON_INDEX THEN
		ROLLBACK;
		RSP := 'ERROR: el ciutada: '||v_ciutada||' ja existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- no existencia de codis amb clau forana a la taula corresponent
	WHEN e_codiNoExistent THEN
		ROLLBACK;
                CASE
                  WHEN INSTR(UPPER(SQLERRM), 'FK1_CIUTADAPAIS')>0 THEN
                    RSP := 'ERROR: El codi de pais no existeix.';
                  WHEN INSTR(UPPER(SQLERRM), 'FK2_CIUTADAREGIO')>0 THEN
                    RSP := 'ERROR: El codi de regio no existeix.';
                  WHEN INSTR(UPPER(SQLERRM), 'FK3_CIUTADALOCALITAT')>0 THEN
                    RSP := 'ERROR: El codi de localitat no existeix.';
                  ELSE
                    RSP := 'ERROR: codi no existent.';
                END CASE;  
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
                -- control dels codis d'error de les dates, son tots els codis ORACLE del -01800 al -01899
                IF SQLCODE > -1900 and SQLCODE <= -1800 THEN
                  RSP := 'ERROR: la data de naixement no es correcte, format data (dd/mm/aaaa)';
                ELSE  
                  RSP := 'ERROR En la inserci del ciutada:'||SQLERRM;
                END IF;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
END p_ciutada_alta;
/

CREATE OR REPLACE PROCEDURE  p_ciutada_baixa(	
	v_idCiutada IN ciutada.idCiutada%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_ciutada_baixa';
        var_par_entrada VARCHAR2(400);
        e_parmEntrada EXCEPTION;
        v_dataNaixement ciutada.datanaixement%TYPE;
BEGIN
    var_par_entrada := 'idCiutada:'||v_idCiutada;
    
    -- validaci parametres entrada
    IF v_idCiutada is null THEN
      RSP := 'ERROR: s''ha d''especificar el codi de ciutada a eliminar';
      RAISE e_parmEntrada;
    END IF;  

    SELECT datanaixement INTO v_datanaixement FROM ciutada WHERE idCiutada=v_idCiutada;

    -- Fi validaci parametres entrada
    -- Eliminacio
    /*  quan eliminem un ciutada, tamb s'eliminen les relacions d'aquest amb els censos a on pot ester incrit (PADROCENS),
        i les votacions que ha pogut realitzar (PARTICIPACIO) */
    DELETE FROM ciutada WHERE idciutada=v_idCiutada;
    -- eliminacio correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- no existeix el ciutada que es vol esborrar
	WHEN NO_DATA_FOUND THEN
		ROLLBACK;
		RSP := 'ERROR: el ciutada : '||v_idCiutada||' no existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
      	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
		RSP := 'ERROR En l''eliminaci del ciutada: '||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
END p_ciutada_baixa;
/

CREATE OR REPLACE PROCEDURE  p_ciutada_modificacio(	
	v_idCiutada IN ciutada.idCiutada%TYPE,
        v_pais IN ciutada.idPais%TYPE,
        v_regio IN ciutada.idRegio%TYPE,
        v_localitat IN ciutada.idLocalitat%TYPE,
        v_dataNaixement IN VARCHAR2,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_ciutada_modificacio';
        var_par_entrada VARCHAR2(400);
        var_dataNaix ciutada.datanaixement%TYPE;
        e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
BEGIN
    -- validaci parametres entrada
    var_par_entrada := 'idCiutada:'||v_idCiutada||' Pais:'||v_pais||' Regio:'||v_regio||' Localitat:'||v_localitat||' DataNaixement:'||v_dataNaixement;
    CASE
      WHEN v_idCiutada is null THEN
	RSP := 'ERROR: Es obligatori especificar el codi del ciutada a modificar.';
        RAISE e_parmEntrada;
      WHEN v_pais is null or v_regio is null or v_localitat is null THEN
	RSP := 'ERROR: Es obligatori especificar el pais, regio i localitat del ciutada.';
        RAISE e_parmEntrada;
      WHEN v_dataNaixement is null THEN
	RSP := 'ERROR: Es obligatori especificar la data de naixement del ciutada.';
        RAISE e_parmEntrada;
      ELSE  
	RSP := 'OK';
    END CASE;
    -- Fi validaci parametres entrada
    SELECT datanaixement INTO var_datanaix FROM ciutada WHERE idCiutada=v_idCiutada;
    -- Update a taula ciutada
    UPDATE ciutada SET idpais=v_pais, idregio=v_regio, idlocalitat=v_localitat, dataNaixement=v_datanaixement WHERE idCiutada=v_idCiutada;
    -- modificaci correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- no existeix el ciutada que es vol modificar
	WHEN NO_DATA_FOUND THEN
		ROLLBACK;
		RSP := 'ERROR: el ciutada : '||v_idCiutada||' no existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- no existencia de codis amb clau forana a la taula corresponent
	WHEN e_codiNoExistent THEN
		ROLLBACK;
                CASE
                  WHEN INSTR(UPPER(SQLERRM), 'FK1_CIUTADAPAIS')>0 THEN
                    RSP := 'ERROR: El codi de pais no existeix.';
                  WHEN INSTR(UPPER(SQLERRM), 'FK2_CIUTADAREGIO')>0 THEN
                    RSP := 'ERROR: El codi de regio no existeix.';
                  WHEN INSTR(UPPER(SQLERRM), 'FK3_CIUTADALOCALITAT')>0 THEN
                    RSP := 'ERROR: El codi de localitat no existeix.';
                  ELSE
                    RSP := 'ERROR: codi no existent.';
                END CASE;  
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
                -- control dels codis d'error de les dates, son tots els codis ORACLE del -01800 al -01899
                IF SQLCODE > -1900 and SQLCODE <= -1800 THEN
                  RSP := 'ERROR: la data de naixement no es correcte, format data (dd/mm/aaaa)';
                ELSE  
                  RSP := 'ERROR En la modificaci del ciutada:'||SQLERRM;
                END IF;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
END p_ciutada_modificacio;
/

CREATE OR REPLACE PROCEDURE  p_ciutada_assigna_cens(	
	v_idCiutada IN ciutada.idCiutada%TYPE,
        v_idCens IN cens.idCens%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_ciutada_assigna_cens';
        var_par_entrada VARCHAR2(400);
        var_numCiutada NUMBER := 0;
        var_numCens NUMBER := 0;
        e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
BEGIN
    var_par_entrada := 'idCiutada:'||v_idCiutada||' idCens:'||v_idCens;
    -- validaci parametres entrada
    IF v_idCiutada is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi del Ciutada.';
      RAISE e_parmEntrada;
    END IF;  
    IF v_idCens is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi del Cens.';
      RAISE e_parmEntrada;
    END IF;  
    -- Fi validaci parametres entrada
    -- Insert a taula padrocens
    INSERT INTO padrocens(idCens, idCiutada) VALUES ( v_idCens,v_idCiutada);
    -- inserci correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;

-- control d'excepcions
EXCEPTION
	-- valors duplicats en una columna amb ndex nic
	WHEN DUP_VAL_ON_INDEX THEN
		ROLLBACK;
		RSP := 'ERROR: el ciutada ja existeix en el cens.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- no existencia de codis amb clau forana a la taula corresponent
	WHEN e_codiNoExistent THEN
		ROLLBACK;
                CASE
                  WHEN INSTR(UPPER(SQLERRM), 'FK1_PADROCENSIDCENS')>0 THEN
                    RSP := 'ERROR: El codi de cens no existeix.';
                  WHEN INSTR(UPPER(SQLERRM), 'FK2_PADROCENSIDCIUTADA')>0 THEN
                    RSP := 'ERROR: El codi de ciutada no existeix.';
                  ELSE
                    RSP := 'ERROR: codi no existent.';
                END CASE;  
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
      	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
                RSP := 'ERROR En l''assignaci del ciutada en el cens :'||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;

END p_ciutada_assigna_cens;
/

CREATE OR REPLACE PROCEDURE  p_ciutada_elimina_cens(	
	v_idCiutada IN ciutada.idCiutada%TYPE,
        v_idCens IN cens.idCens%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_ciutada_elimina_cens';
        var_par_entrada VARCHAR2(400);
        var_idCiutada padrocens.idCiutada%TYPE;
        var_numCiutada NUMBER := 0;
        var_numCens NUMBER := 0;
        e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
BEGIN
    var_par_entrada := 'idCiutada:'||v_idCiutada||' idCens:'||v_idCens;

    -- validaci parametres entrada
    IF v_idCiutada is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi del Ciutada.';
      RAISE e_parmEntrada;
    END IF;  
    IF v_idCens is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi del Cens.';
      RAISE e_parmEntrada;
    END IF;  
    
    SELECT idciutada INTO var_idCiutada FROM padrocens WHERE idCens=v_idCens AND idCiutada=v_idCiutada;
    -- Fi validaci parametres entrada

    -- Eliminacio
    DELETE FROM padrocens WHERE idCens=v_idCens AND idciutada=v_idCiutada;
    -- eliminacio correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;

-- control d'excepcions
EXCEPTION
	-- no existeix el ciutada que es vol esborrar
	WHEN NO_DATA_FOUND THEN
		ROLLBACK;
		RSP := 'ERROR: el ciutada : '||v_idCiutada||' no existeix en el cens '||v_idCens;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
      	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
		RSP := 'ERROR En l''eliminaci del ciutada en el cens: '||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;

END p_ciutada_elimina_cens;
/



