CREATE OR REPLACE PROCEDURE  p_votacio_alta(	
	v_titol IN votacio.titol%TYPE,
        v_descripcio IN votacio.descripcio%TYPE,
        v_dataInici IN VARCHAR2,
        v_dataFinal IN VARCHAR2,
        v_dataPublicacio IN VARCHAR2,
        v_presidentMesa IN votacio.presidentmesa%TYPE,
        v_urlWeb IN votacio.urlweb%TYPE,
        v_estat IN votacio.estat%TYPE,
        v_idOpcio1 IN opcionsvotacio.idOpcio%TYPE,
        v_idOpcio2 IN opcionsvotacio.idOpcio%TYPE,
        v_idCreat OUT votacio.idvotacio%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_votacio_alta';
        var_par_entrada VARCHAR2(400);
        var_numOpcio NUMBER;
        var_dataInici votacio.datainici%TYPE;
        var_dataFinal votacio.datafinal%TYPE;
        var_dataPublicacio votacio.datapublicacio%TYPE;
        e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
BEGIN
    var_par_entrada := 'Titol:'||v_titol||' Descripcio:'||v_descripcio||' Data Inici:'||v_dataInici;
    var_par_entrada := var_par_entrada||' Data Final:'||v_dataFinal||' Data Publicacio:'||v_dataPublicacio;
    var_par_entrada := var_par_entrada||' President mesa:'||v_presidentMesa||' Url Web:'||v_urlWeb||' Estat:'||v_estat;
    var_par_entrada := var_par_entrada||' Opci 1:'||v_idOpcio1||' Opci 2:'||v_idOpcio2;

    -- validaci parametres entrada
    CASE
      WHEN v_titol is null THEN
	RSP := 'ERROR: Es obligatori especificar el titol de la votaci.';
        RAISE e_parmEntrada;
      WHEN v_dataInici is null THEN
	RSP := 'ERROR: Es obligatori especificar la data d''inici de la votaci.';
        RAISE e_parmEntrada;
      WHEN v_dataFinal is null THEN
	RSP := 'ERROR: Es obligatori especificar la data final de la votaci.';
        RAISE e_parmEntrada;
      WHEN v_dataPublicacio is null THEN
	RSP := 'ERROR: Es obligatori especificar la data de publicaci de la votaci.';
        RAISE e_parmEntrada;
      WHEN v_presidentMesa is null THEN
	RSP := 'ERROR: Es obligatori especificar el president de la Mesa.';
        RAISE e_parmEntrada;
      WHEN v_estat is null THEN
	RSP := 'ERROR: Es obligatori especificar l''estat de la votaci.';
        RAISE e_parmEntrada;
      WHEN v_idOpcio1 is null or v_idOpcio2 is null THEN
	RSP := 'ERROR: Es obligatori especificar les dues opcions que com a mnim ha de tenir la votaci.';
        RAISE e_parmEntrada;
      WHEN v_estat = 'F' THEN
	RSP := 'ERROR: No es pot crear una votaci amb l''estat F de finalitzada.';
        RAISE e_parmEntrada;
      ELSE  
	RSP := 'OK';
    END CASE;
    
    -- valida que sigui correcte la data inici
    BEGIN
      var_dataInici:=to_date(v_datainici);
    EXCEPTION
	WHEN OTHERS THEN
          RSP := 'ERROR: La data d''inici no es correcte, format data (dd/mm/aaaa).';
          RAISE e_parmEntrada;
    END;    
    -- valida que sigui correcte la data final
    BEGIN
      var_dataFinal:=to_date(v_datafinal);
    EXCEPTION
	WHEN OTHERS THEN
          RSP := 'ERROR: La data final no es correcte, format data (dd/mm/aaaa).';
          RAISE e_parmEntrada;
    END;    
    -- valida que sigui correcte la data publicacio
    BEGIN
      var_dataPublicacio:=to_date(v_datapublicacio);
    EXCEPTION
	WHEN OTHERS THEN
          RSP := 'ERROR: La data de publicacio no es correcte, format data (dd/mm/aaaa).';
          RAISE e_parmEntrada;
    END;
    IF var_dataFinal < var_dataInici THEN
       RSP := 'ERROR: La data final ha de ser posterior o igual a la data d''inici';
       RAISE e_parmEntrada;
    END IF;
    IF var_dataPublicacio > var_dataInici-7 THEN
       RSP := 'ERROR: La data de publicacio ha de ser com a minim d''una setmana abans que la data d''inici.';
       RAISE e_parmEntrada;
    END IF;
    
    -- validaci existencia opcins 1 i 2
    SELECT count(*) INTO var_numOpcio FROM opcio WHERE idOpcio=v_idOpcio1;
    IF var_numOpcio = 0 THEN
      RSP := 'ERROR: l''opci : '||v_idOpcio1||' no existeix.';
      RAISE e_parmEntrada;
    END IF;  
    SELECT count(*) INTO var_numOpcio FROM opcio WHERE idOpcio=v_idOpcio2;
    IF var_numOpcio = 0 THEN
      RSP := 'ERROR: l''opci : '||v_idOpcio2||' no existeix.';
      RAISE e_parmEntrada;
    END IF;  
    
    -- Fi validaci parametres entrada
    -- Insert a votacio
    INSERT INTO votacio(titol,descripcio,datainici,datafinal,datapublicacio,presidentmesa,urlweb,estat,npersonesdretvot,nvotstotalsemesos,percenparticipacio)
      VALUES (v_titol,v_descripcio,var_datainici,var_datafinal,var_datapublicacio,v_presidentmesa,v_urlweb,v_estat,0,0,0)
      RETURNING idvotacio INTO v_idCreat;
      
    -- Insert a taula opcionsvotacio
    INSERT INTO opcionsvotacio(idVotacio, idOpcio, nnumerovots, percenvots) VALUES ( v_idCreat,v_idOpcio1,0,0);
    INSERT INTO opcionsvotacio(idVotacio, idOpcio, nnumerovots, percenvots) VALUES ( v_idCreat,v_idOpcio2,0,0);
      
    -- inserci correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' || RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- valors duplicats en una columna amb ndex nic
	WHEN DUP_VAL_ON_INDEX THEN
		ROLLBACK;
		RSP := 'ERROR: la votacio: '||v_titol||' ja existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' ||RSP);
		COMMIT;
       	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' ||RSP);
		COMMIT;
       	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
                RSP := 'ERROR En la inserci de la votaci:'||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' ||RSP);
		COMMIT;
END p_votacio_alta;
/

CREATE OR REPLACE PROCEDURE  p_votacio_baixa(	
	v_idVotacio IN votacio.idVotacio%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_votacio_baixa';
        var_par_entrada VARCHAR2(400);
        e_parmEntrada EXCEPTION;
        var_estat votacio.estat%TYPE;
BEGIN
    var_par_entrada := 'idVotacio:'||v_idVotacio;

--    -- validaci parametres entrada

    IF v_idVotacio is null THEN
      RSP := 'ERROR: Es obligatori especificar el codi de la votacio a eliminar.';
      RAISE e_parmEntrada;
    END IF;

    SELECT estat INTO var_estat FROM votacio WHERE idVotacio=v_idVotacio;

    -- no es pot eliminar una votaci que ja s'ha finalitzat.
    IF var_estat = 'F' THEN
	RSP := 'ERROR: No es pot eliminar una votaci que t l''estat de finalitzada (Estat=F).';
        RAISE e_parmEntrada;
    END IF;
    -- Fi validaci parametres entrada

    -- Eliminacio
    /*  eliminat la votaci, les taules OPCIONSVOTACIO i CENSVOTACIO tenen les seves claus foranes amb la taula VOTACIO
        amb DELETE CASCADE, d'aquesta manera, quan eliminem una votaci, tamb s'eliminen totes les opcions que pot tenir
        assignades, i els censos que pot tenir assignats */
    DELETE FROM votacio WHERE idVotacio=v_idVotacio;
    -- eliminacio correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- no existeix la votacio que es vol esborrar
	WHEN NO_DATA_FOUND THEN
		ROLLBACK;
		RSP := 'ERROR: la votaci : '||v_idVotacio||' no existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
      	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
		RSP := 'ERROR En l''eliminaci de la votaci: '||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
END p_votacio_baixa;
/

CREATE OR REPLACE PROCEDURE  p_votacio_modificacio(	
	v_idVotacio IN votacio.idVotacio%TYPE,
      	v_titol IN votacio.titol%TYPE,
        v_descripcio IN votacio.descripcio%TYPE,
        v_dataInici IN VARCHAR2,
        v_dataFinal IN VARCHAR2,
        v_dataPublicacio IN VARCHAR2,
        v_presidentMesa IN votacio.presidentmesa%TYPE,
        v_urlWeb IN votacio.urlweb%TYPE,
        v_estat IN votacio.estat%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_votacio_modificacio';
        var_par_entrada VARCHAR2(400);
        var_dataInici votacio.datainici%TYPE;
        var_dataFinal votacio.datafinal%TYPE;
        var_dataPublicacio votacio.datapublicacio%TYPE;
        var_estat votacio.estat%TYPE;
        e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
BEGIN

    var_par_entrada := 'idVotacio:'||v_idVotacio||'Titol:'||v_titol||' Descripcio:'||v_descripcio||' Data Inici:'||v_dataInici;
    var_par_entrada := var_par_entrada||' Data Final:'||v_dataFinal||' Data Publicacio:'||v_dataPublicacio;
    var_par_entrada := var_par_entrada||' President mesa:'||v_presidentMesa||' Url Web:'||v_urlWeb||' Estat:'||v_estat;

    -- validaci parametres entrada
    CASE
      WHEN v_idVotacio is null THEN
	RSP := 'ERROR: Es obligatori especificar el codi de la votaci.';
        RAISE e_parmEntrada;
      WHEN v_titol is null THEN
	RSP := 'ERROR: Es obligatori especificar el titol de la votaci.';
        RAISE e_parmEntrada;
      WHEN v_dataInici is null THEN
	RSP := 'ERROR: Es obligatori especificar la data d''inici de la votaci.';
        RAISE e_parmEntrada;
      WHEN v_dataFinal is null THEN
	RSP := 'ERROR: Es obligatori especificar la data final de la votaci.';
        RAISE e_parmEntrada;
      WHEN v_dataPublicacio is null THEN
	RSP := 'ERROR: Es obligatori especificar la data de publicaci de la votaci.';
        RAISE e_parmEntrada;
      WHEN v_presidentMesa is null THEN
	RSP := 'ERROR: Es obligatori especificar el president de la Mesa.';
        RAISE e_parmEntrada;
      WHEN v_estat is null THEN
	RSP := 'ERROR: Es obligatori especificar l''estat de la votaci.';
        RAISE e_parmEntrada;
      WHEN v_estat = 'F' THEN
	RSP := 'ERROR: No es pot modificar l''estat, i posar l''estat F de finalitzada.';
        RAISE e_parmEntrada;
      ELSE  
	RSP := 'OK';
    END CASE;
    
    -- valida que sigui correcte la data inici
    BEGIN
      var_dataInici:=to_date(v_datainici);
    EXCEPTION
	WHEN OTHERS THEN
          RSP := 'ERROR: La data d''inici no es correcte, format data (dd/mm/aaaa).';
          RAISE e_parmEntrada;
    END;    
    -- valida que sigui correcte la data final
    BEGIN
      var_dataFinal:=to_date(v_datafinal);
    EXCEPTION
	WHEN OTHERS THEN
          RSP := 'ERROR: La data final no es correcte, format data (dd/mm/aaaa).';
          RAISE e_parmEntrada;
    END;    
    -- valida que sigui correcte la data publicacio
    BEGIN
      var_dataPublicacio:=to_date(v_datapublicacio);
    EXCEPTION
	WHEN OTHERS THEN
          RSP := 'ERROR: La data de publicacio no es correcte, format data (dd/mm/aaaa).';
          RAISE e_parmEntrada;
    END;
    IF var_dataFinal < var_dataInici THEN
       RSP := 'ERROR: La data final ha de ser posterior o igual a la data d''inici';
       RAISE e_parmEntrada;
    END IF;
    IF var_dataPublicacio > var_dataInici-7 THEN
       RSP := 'ERROR: La data de publicacio ha de ser com a minim d''una setmana abans que la data d''inici.';
       RAISE e_parmEntrada;
    END IF;
    -- Fi validaci parametres entrada
    SELECT estat INTO var_estat FROM votacio WHERE idVotacio=v_idVotacio;

    -- no es pot modificar una votaci que ja s'ha finalitzat.
    IF var_estat = 'F' THEN
	RSP := 'ERROR: No es pot modificar una votaci que t l''estat de finalitzada (Estat=F).';
        RAISE e_parmEntrada;
    END IF;
    
    -- Update a taula votacio
    UPDATE votacio SET  titol=v_titol, 
                        descripcio=v_descripcio, 
                        datainici=var_dataInici,
                        datafinal=var_dataFinal,
                        dataPublicacio=var_dataPublicacio,
                        presidentMesa=v_presidentMesa,
                        urlweb=v_urlweb,
                        estat=v_estat WHERE idVotacio=v_idVotacio;
    -- modificaci correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- valors duplicats en una columna amb ndex nic
	WHEN DUP_VAL_ON_INDEX THEN
		ROLLBACK;
		RSP := 'ERROR: la votacio: '||v_titol||' ja existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- no existeix la votaci que es vol modificar
	WHEN NO_DATA_FOUND THEN
		ROLLBACK;
		RSP := 'ERROR: la votaci : '||v_idVotacio||' no existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
       	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
       	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
                RSP := 'ERROR En la modificacio de la votaci:'||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
END p_votacio_modificacio;
/

CREATE OR REPLACE PROCEDURE  p_votacio_afegir_opcio(	
	v_idVotacio IN opcionsvotacio.idVotacio%TYPE,
        v_idOpcio IN opcionsvotacio.idOpcio%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_votacio_afegir_opcio';
        var_par_entrada VARCHAR2(400);
        var_numVotacio NUMBER := 0;
        var_numOpcio NUMBER := 0;
        e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
BEGIN
    var_par_entrada := 'idVotacio:'||v_idVotacio||' idOpcio:'||v_idOpcio;
    -- validaci parametres entrada
    IF v_idVotacio is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi de la Votaci.';
      RAISE e_parmEntrada;
    END IF;  
    IF v_idOpcio is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi de l''Opci.';
      RAISE e_parmEntrada;
    END IF;  
    -- Fi validaci parametres entrada
    -- Insert a taula opcionsvotacio
    INSERT INTO opcionsvotacio(idVotacio, idOpcio,nnumerovots, percenvots) VALUES ( v_idVotacio,v_idOpcio,0,0);
    -- inserci correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;

-- control d'excepcions
EXCEPTION
	-- valors duplicats en una columna amb ndex nic
	WHEN DUP_VAL_ON_INDEX THEN
		ROLLBACK;
		RSP := 'ERROR: l''opci ja est assignada a aquesta votaci.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- no existencia de codis amb clau forana a la taula corresponent
	WHEN e_codiNoExistent THEN
		ROLLBACK;
                CASE
                  WHEN INSTR(UPPER(SQLERRM), 'FK1_OPCIONSVOTACIOIDVOTACIO')>0 THEN
                    RSP := 'ERROR: El codi de votaci no existeix.';
                  WHEN INSTR(UPPER(SQLERRM), 'FK2_OPCIONSVOTACIOIDOPCIO')>0 THEN
                    RSP := 'ERROR: El codi d''opci no existeix.';
                  ELSE
                    RSP := 'ERROR: codi no existent.';
                END CASE;  
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
      	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
                RSP := 'ERROR En la assignaci de l''opci a la votaci :'||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;

END p_votacio_afegir_opcio;
/

CREATE OR REPLACE PROCEDURE  p_votacio_elimina_opcio(	
	v_idVotacio IN opcionsvotacio.idVotacio%TYPE,
        v_idOpcio IN opcionsvotacio.idOpcio%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_votacio_elimina_opcio';
        var_par_entrada VARCHAR2(400);
        var_numOpcio NUMBER;
        var_idOpcio opcionsvotacio.idOpcio%TYPE;
        e_parmEntrada EXCEPTION;
BEGIN
    var_par_entrada := 'idVotacio: '||v_idVotacio||' idOpcio:'||v_idOpcio;

    -- validaci parametres entrada
    IF v_idVotacio is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi de la Votaci.';
      RAISE e_parmEntrada;
    END IF;  
    IF v_idOpcio is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi de l''Opci.';
      RAISE e_parmEntrada;
    END IF;  
    
    SELECT idOpcio INTO var_idOpcio FROM opcionsvotacio WHERE idVotacio=v_idVotacio AND idOpcio=v_idOpcio;
    
    SELECT count(*) INTO var_numOpcio FROM opcionsvotacio WHERE idVotacio=v_idVotacio;
    IF var_numOpcio = 2 THEN
      RSP := 'ERROR: La votaci com a mnim ha de tenir 2 opcions, no es pot eliminar.';
      RAISE e_parmEntrada;
    END IF;  
        
    -- Fi validaci parametres entrada

    -- Eliminacio
    DELETE FROM opcionsvotacio WHERE idVotacio=v_idVotacio AND idOpcio=v_idOpcio;
    -- eliminacio correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;

-- control d'excepcions
EXCEPTION
	-- no existeix l'opcio que es vol esborrar en la votacio
	WHEN NO_DATA_FOUND THEN
		ROLLBACK;
		RSP := 'ERROR: l''opcio : '||v_idOpcio||' no existeix en la votaci '||v_idVotacio;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
      	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
		RSP := 'ERROR En l''eliminaci de l''opci-votaci: '||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;

END p_votacio_elimina_opcio;
/

CREATE OR REPLACE PROCEDURE  p_votacio_afegir_cens(	
	v_idVotacio IN censvotacio.idVotacio%TYPE,
        v_idCens IN censvotacio.idCens%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_votacio_afegir_cens';
        var_par_entrada VARCHAR2(400);
        e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
BEGIN
    var_par_entrada := 'idVotacio:'||v_idVotacio||' idCens:'||v_idCens;
    -- validaci parametres entrada
    IF v_idVotacio is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi de la Votaci.';
      RAISE e_parmEntrada;
    END IF;  
    IF v_idCens is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi del Cens.';
      RAISE e_parmEntrada;
    END IF;  
    -- Fi validaci parametres entrada
    -- Insert a taula censvotacio
    INSERT INTO censvotacio(idVotacio, idCens) VALUES ( v_idVotacio,v_idCens);
    -- inserci correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;

-- control d'excepcions
EXCEPTION
	-- valors duplicats en una columna amb ndex nic
	WHEN DUP_VAL_ON_INDEX THEN
		ROLLBACK;
		RSP := 'ERROR: el Cens ja est assignat a aquesta votaci.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- no existencia de codis amb clau forana a la taula corresponent
	WHEN e_codiNoExistent THEN
		ROLLBACK;
                CASE
                  WHEN INSTR(UPPER(SQLERRM), 'FK1_CENSVOTACIOIDVOTACIO')>0 THEN
                    RSP := 'ERROR: El codi de votaci no existeix.';
                  WHEN INSTR(UPPER(SQLERRM), 'FK2_CENSVOTACIOIDCENS')>0 THEN
                    RSP := 'ERROR: El codi de cens no existeix.';
                  ELSE
                    RSP := 'ERROR: codi no existent.';
                END CASE;  
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
      	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
                RSP := 'ERROR En afegir el cens a la votaci :'||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;

END p_votacio_afegir_cens;
/

CREATE OR REPLACE PROCEDURE  p_votacio_elimina_cens(	
	v_idVotacio IN censvotacio.idVotacio%TYPE,
        v_idCens IN censvotacio.idCens%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_votacio_elimina_cens';
        var_par_entrada VARCHAR2(400);
        var_idCens censvotacio.idCens%TYPE;
        e_parmEntrada EXCEPTION;
BEGIN
    var_par_entrada := 'idVotacio: '||v_idVotacio||' idCens:'||v_idCens;

    -- validaci parametres entrada
    IF v_idVotacio is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi de la Votaci.';
      RAISE e_parmEntrada;
    END IF;  
    IF v_idCens is NULL THEN
      RSP := 'ERROR: Es obligatori especificar el codi del Cens.';
      RAISE e_parmEntrada;
    END IF;  
    
    SELECT idCens INTO var_idCens FROM censvotacio WHERE idVotacio=v_idVotacio AND idCens=v_idCens;
    -- Fi validaci parametres entrada

    -- Eliminacio
    DELETE FROM censvotacio WHERE idVotacio=v_idVotacio AND idCens=v_idCens;
    -- eliminacio correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;

-- control d'excepcions
EXCEPTION
	-- no existeix l'opcio que es vol esborrar en la votacio
	WHEN NO_DATA_FOUND THEN
		ROLLBACK;
		RSP := 'ERROR: el cens : '||v_idCens||' no existeix en la votaci '||v_idVotacio;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
      	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
		RSP := 'ERROR En l''eliminaci del cens-votaci: '||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;

END p_votacio_elimina_cens;
/

