CREATE OR REPLACE PROCEDURE  p_opcio_alta(	
	v_nom IN opcio.nom%TYPE,
        v_idCreat OUT opcio.idopcio%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_opcio_alta';
        var_par_entrada VARCHAR2(400);
        e_parmEntrada EXCEPTION;
BEGIN
    var_par_entrada := 'Nom:'||v_nom;
    -- validaci parametres entrada
    IF v_Nom is NULL then 
      RSP := 'ERROR: Es obligatori especificar el nom de l''opci.';
      RAISE e_parmEntrada;
    END IF;
    -- Fi validaci parametres entrada
    -- Insert a taula opcio
    INSERT INTO opcio(nom) 
      VALUES ( v_nom)
      RETURNING idopcio INTO v_idCreat;

    -- inserci correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' ||RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- valors duplicats en una columna amb ndex nic
	WHEN DUP_VAL_ON_INDEX THEN
		ROLLBACK;
		RSP := 'ERROR: l''opci : '||v_nom||' ja existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' ||RSP);
		COMMIT;
	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' ||RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
                RSP := 'ERROR En la inserci de l''opci :'||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, v_idCreat || ' - ' ||RSP);
		COMMIT;
END p_opcio_alta;
/

CREATE OR REPLACE PROCEDURE  p_opcio_baixa(	
	v_idOpcio IN opcio.idOpcio%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_opcio_baixa';
        var_par_entrada VARCHAR2(400);
        var_nom opcio.nom%TYPE;
        e_parmEntrada EXCEPTION;
BEGIN
    var_par_entrada := 'idOpcio:'||v_idOpcio;

    -- validaci parametres entrada
    IF v_idOpcio is NULL then 
      RSP := 'ERROR: Es obligatori especificar el codi de l''opci a eliminar.';
      RAISE e_parmEntrada;
    END IF;
    SELECT nom INTO var_nom FROM opcio WHERE idOpcio=v_idOpcio;
    -- Fi validaci parametres entrada

    -- Eliminacio
    DELETE FROM opcio WHERE idOpcio=v_idOpcio;
    -- eliminacio correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- no existeix l'opci que es vol esborrar
	WHEN NO_DATA_FOUND THEN
		ROLLBACK;
		RSP := 'ERROR: l''opci : '||v_idOpcio||' no existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
		RSP := 'ERROR En l''eliminaci de l''opci:'||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
END p_opcio_baixa;
/

CREATE OR REPLACE PROCEDURE  p_opcio_modificacio(	
        v_idOpcio IN opcio.idOpcio%TYPE,
	v_nom IN opcio.nom%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_opcio_modificacio';
        var_par_entrada VARCHAR2(400);
        var_nom opcio.nom%TYPE;
        e_parmEntrada EXCEPTION;
BEGIN

    var_par_entrada := 'idOpcio:'||v_idOpcio||' nom:'||v_nom;

    -- validaci parametres entrada
    CASE
      WHEN v_idOPcio is null THEN
	RSP := 'ERROR: Es obligatori especificar el codi de l''opci.';
        RAISE e_parmEntrada;
      WHEN v_nom is null THEN
	RSP := 'ERROR: Es obligatori especificar el nom de l''opci.';
        RAISE e_parmEntrada;
      ELSE  
	RSP := 'OK';
    END CASE;
    -- Fi validaci parametres entrada
    SELECT nom INTO var_nom FROM opcio WHERE idOpcio=v_idOpcio;
    -- Update a taula opcio
    UPDATE opcio SET  nom=v_nom WHERE idOpcio=v_idOpcio;
    -- modificaci correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- valors duplicats en una columna amb ndex nic
	WHEN DUP_VAL_ON_INDEX THEN
		ROLLBACK;
		RSP := 'ERROR: l''opci : '||v_nom||' ja existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
	-- no existeix l'opci que es vol modificar
	WHEN NO_DATA_FOUND THEN
		ROLLBACK;
		RSP := 'ERROR: l''opci: '||v_idOpcio||' no existeix.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
       	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
       	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
                RSP := 'ERROR En la modificaci de l''opci:'||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
END p_opcio_modificacio;
/
