CREATE OR REPLACE PROCEDURE  p_votar(
        v_idCiutada IN participacio.idCiutada%TYPE,
        v_idVotacio IN participacio.idVotacio%TYPE,
        v_idOpcio IN opcionsvotacio.idOpcio%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_votar';
        var_par_entrada VARCHAR2(400);
        var_datanaixement ciutada.datanaixement%TYPE;
        var_MajoriaEdat NUMBER := 18;
        var_numCiutada NUMBER;
        var_numOpcio NUMBER;
        var_numVotacio NUMBER;
        var_dataIniciVotacio votacio.datainici%TYPE;
        var_dataFinalVotacio votacio.datafinal%TYPE;
        e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
BEGIN

    var_par_entrada := 'idCiutada:'||v_idCiutada||' idVotacio:'||v_idVotacio||' idOpcio:'||v_idOpcio;

    -- validaci parametres entrada
    CASE
      WHEN v_idCiutada is null THEN
	RSP := 'ERROR: Es obligatori especificar el ciutada que realitza el vot.';
        RAISE e_parmEntrada;
      WHEN v_idVotacio is null THEN
	RSP := 'ERROR: Es obligatori especificar la votaci que es vota.';
        RAISE e_parmEntrada;
      WHEN v_idOpcio is null THEN
	RSP := 'ERROR: Es obligatori especificar l''opci que es vota.';
        RAISE e_parmEntrada;
      ELSE  
	RSP := 'OK';
    END CASE;

    -- validaci existeix ciutada
    SELECT count(*) INTO var_numCiutada FROM ciutada WHERE idCiutada=v_idCiutada;
    IF var_numCiutada = 0 THEN
      RSP := 'ERROR: el ciutada : '||v_idCiutada||' no existeix.';
      RAISE e_parmEntrada;
    END IF;  
    
    -- validaci que el ciutada es major d'edat
    SELECT datanaixement INTO var_datanaixement FROM ciutada where idCiutada=v_idCiutada;
    IF var_datanaixement > add_months(sysdate, -(var_MajoriaEdat*12)) THEN
      RSP := 'ERROR: el ciutada : '||v_idCiutada||' no es major d''edat, no pot votar.';
      RAISE e_parmEntrada;
    END IF;  

    -- validaci que el ciutada pertany a un dels cens assignats a la votacio
    var_numCiutada := 0;
    SELECT count(*) INTO var_numCiutada 
        FROM padrocens 
            INNER JOIN ciutada ON padrocens.idCiutada=ciutada.idCiutada
            INNER JOIN censvotacio ON padrocens.idcens=censvotacio.idcens
        WHERE ciutada.idCiutada=v_idCiutada and censvotacio.idvotacio=v_idVotacio;
    IF var_numCiutada = 0 THEN
      RSP := 'ERROR: el ciutada : '||v_idCiutada||' no pertany a cap dels cens assignats a la votaci.';
      RAISE e_parmEntrada;
    END IF;  

    -- validaci existencia opci en la votaci
    SELECT count(*) INTO var_numOpcio FROM opcionsvotacio WHERE idVotacio=v_idVotacio AND idOpcio=v_idOpcio;
    IF var_numOpcio = 0 THEN
      RSP := 'ERROR: l''opcio : '||v_idOpcio||' no esta disponible per votar-la.';
      RAISE e_parmEntrada;
    END IF;  

    -- validaci existeix votacio
    SELECT count(*) INTO var_numVotacio FROM votacio WHERE idVotacio=v_idVotacio;
    IF var_numVotacio = 0 THEN
      RSP := 'ERROR: la votaci: '||v_idVotacio||' no existeix.';
      RAISE e_parmEntrada;
    END IF;  
    
    -- validaci que la votaci estigui oberta
    SELECT  datainici INTO var_dataIniciVotacio
      FROM votacio where idVotacio=v_idVotacio;
    SELECT  datafinal INTO var_dataFinalVotacio 
      FROM votacio where idVotacio=v_idVotacio;
    CASE
      WHEN var_dataIniciVotacio > sysdate THEN
	RSP := 'ERROR: La votaci encara no esta oberta.';
        RAISE e_parmEntrada;
      WHEN var_dataFinalVotacio < sysdate THEN
	RSP := 'ERROR: La votaci ja esta tancada, no admet ms vots.';
        RAISE e_parmEntrada;
      ELSE  
	RSP := 'OK';
    END CASE;
    -- Fi validaci parametres entrada
    -- Insert a participacio
    INSERT INTO participacio(idvotacio, idciutada) VALUES ( v_idVotacio, v_idCiutada);

    -- acumular vot a l'opci
    UPDATE opcionsvotacio SET nnumerovots=nnumerovots+1 WHERE idVotacio=v_idVotacio AND idOpcio=v_idOpcio;

    -- acumular vot emes
    UPDATE votacio SET nvotstotalsemesos=nvotstotalsemesos+1 WHERE idVotacio=v_idVotacio;
    
    -- accio correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
	-- valors duplicats en una columna amb ndex nic
	WHEN DUP_VAL_ON_INDEX THEN
		ROLLBACK;
		RSP := 'ERROR: el ciutada: '||v_idCiutada||' ja ha votat en aquesta votaci.';
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
       	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
       	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
                RSP := 'ERROR En la realitzacio del vot: '||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
END p_votar;
/

CREATE OR REPLACE PROCEDURE  p_final_votacio(	
        v_idVotacio IN participacio.idVotacio%TYPE,
        RSP OUT VARCHAR2
        )
IS
        var_procediment VARCHAR2(50) := 'p_final_votacio';
        var_par_entrada VARCHAR2(400);
        var_numVotacio NUMBER;
        var_estatVotacio votacio.estat%TYPE;
        var_dataFinalVotacio votacio.datafinal%TYPE;
        var_numPersones NUMBER;
        var_numVotsEmesos NUMBER;
        var_idPais pais.idpais%TYPE;
        var_exercici VARCHAR2(4);
        var_numStat NUMBER;
        var_MajoriaEdat NUMBER := 18;
        var_numCiutadans NUMBER;
        var_numCiutadansVotat NUMBER;
        var_idCens censvotacio.idcens%TYPE;
        var_numVotacions NUMBER;
        var_sumaParticipacio NUMBER;
        e_codiNoExistent EXCEPTION;
	PRAGMA EXCEPTION_INIT(e_codiNoExistent, -02291);
        e_parmEntrada EXCEPTION;
        
        -- Cursor de les votacions per pais
        CURSOR c_Paisos_votacio IS
          SELECT pais.idPais 
            FROM pais
              INNER JOIN CENS ON (pais.idpais=cens.idpais AND cens.tipus<>'G') OR cens.tipus='G'
              INNER JOIN CENSVOTACIO ON cens.idcens = censvotacio.idcens
            WHERE censvotacio.idVotacio=v_idVotacio
            GROUP BY pais.idPais;

        -- Cursor dels cens de la votacio
        CURSOR c_cens_votacio IS
          SELECT idCens 
          FROM censvotacio
          WHERE idvotacio = v_idVotacio;
                
BEGIN

    var_par_entrada := ' idVotacio:'||v_idVotacio;
    -- validaci parametres entrada
    IF v_idVotacio is null THEN
      RSP := 'ERROR: Es obligatori especificar la votaci que es vol finalitzar.';
      RAISE e_parmEntrada;
    END IF;
    -- validaci existeix votacio
    SELECT count(*) INTO var_numVotacio FROM votacio WHERE idVotacio=v_idVotacio;
    IF var_numVotacio = 0 THEN
      RSP := 'ERROR: la votaci: '||v_idVotacio||' no existeix.';
      RAISE e_parmEntrada;
    END IF;  
    -- validaci que la votaci hagi acabat
    SELECT  datafinal INTO var_dataFinalVotacio 
      FROM votacio where idVotacio=v_idVotacio;
    IF var_dataFinalVotacio > sysdate THEN
      RSP := 'ERROR: La data final de la votaci es posterior, no es pot finalitzar.';
      RAISE e_parmEntrada;
    END IF;
    SELECT  estat INTO var_estatVotacio 
      FROM votacio where idVotacio=v_idVotacio;
    IF var_estatVotacio = 'F' THEN
      RSP := 'ERROR: La votaci ja s''havia finalitzat anteriorment.';
      RAISE e_parmEntrada;
    END IF;
    -- Fi validaci parametres entrada
    -- calcular persones amb dret a vot (votacio)
    SELECT COUNT(*) INTO var_numPersones 
    FROM (SELECT padrocens.idciutada
            FROM padrocens
              INNER JOIN censvotacio on padrocens.idcens=censvotacio.idcens
            WHERE censvotacio.idVotacio=v_idVotacio
            GROUP BY padrocens.idciutada);    
    -- calcular vots totals emesos (votacio)
    SELECT SUM(nnumerovots) INTO var_numVotsEmesos FROM opcionsvotacio WHERE idVotacio=v_idVotacio;
    -- calcular percentatge participacio (votacio)
    -- posar 'F' a la votacio (votacio)
    UPDATE votacio SET  npersonesdretvot=var_numPersones,
                        nvotstotalsemesos=var_numVotsEmesos,
                        percenparticipacio=ROUND((var_numVotsEmesos/var_numPersones)*100,2),
                        estat='F'
      WHERE idVotacio=v_idVotacio;
    -- calcular percentatge opcions votacio (opcionsvotacio)
    UPDATE opcionsvotacio 
      SET percenvots=ROUND((nnumerovots/var_numVotsEmesos)*100,2) 
      WHERE idVotacio=v_idVotacio;
    -- recalcula valors (stat_any) (Exercici)
    var_exercici := extract(year from var_dataFinalVotacio);
    var_numStat:= 0;
    SELECT count(*) INTO var_numStat
       FROM stat_any
       WHERE exercici=var_exercici;
    IF var_numStat = 0 THEN
      INSERT INTO stat_any(exercici) VALUES ( var_exercici);
    END IF;
    
    -- votacio que mes participacio ha obtingut en aquest any
    UPDATE stat_any
      SET votaciomespart = (SELECT * 
                              FROM (SELECT idvotacio 
                                      FROM votacio 
                                      WHERE extract(year from datafinal)=var_exercici 
                                      ORDER BY percenparticipacio DESC)
                              WHERE rownum=1)
    WHERE exercici = var_exercici;          
         

    -- localitat europea a on han votat mes els seus ciutadans
    UPDATE stat_any
      SET paismesvots = (SELECT pais FROM
                            (SELECT ciutada.idpais as pais, ciutada.idregio as regio, ciutada.idlocalitat as localitat, count(ciutada.idlocalitat) 
                              FROM participacio
                                INNER JOIN ciutada ON participacio.idciutada=ciutada.idCiutada
                                INNER JOIN votacio ON participacio.idvotacio=votacio.idvotacio
                              WHERE extract(year from datafinal)=var_exercici AND estat='F'  
                              GROUP BY ciutada.idpais, ciutada.idregio, ciutada.idlocalitat
                              ORDER BY count(ciutada.idlocalitat) DESC)
                          WHERE ROWNUM=1),
          regiomesvots = (SELECT regio FROM
                            (SELECT ciutada.idpais as pais, ciutada.idregio as regio, ciutada.idlocalitat as localitat, count(ciutada.idlocalitat) 
                              FROM participacio
                                INNER JOIN ciutada ON participacio.idciutada=ciutada.idCiutada
                                INNER JOIN votacio ON participacio.idvotacio=votacio.idvotacio
                              WHERE extract(year from datafinal)=var_exercici AND estat='F'  
                              GROUP BY ciutada.idpais, ciutada.idregio, ciutada.idlocalitat
                              ORDER BY count(ciutada.idlocalitat) DESC)
                          WHERE ROWNUM=1),
          localitatmesvots = (SELECT localitat FROM
                            (SELECT ciutada.idpais as pais, ciutada.idregio as regio, ciutada.idlocalitat as localitat, count(ciutada.idlocalitat) 
                              FROM participacio
                                INNER JOIN ciutada ON participacio.idciutada=ciutada.idCiutada
                                INNER JOIN votacio ON participacio.idvotacio=votacio.idvotacio
                              WHERE extract(year from datafinal)=var_exercici AND estat='F'  
                              GROUP BY ciutada.idpais, ciutada.idregio, ciutada.idlocalitat
                              ORDER BY count(ciutada.idlocalitat) DESC)
                          WHERE ROWNUM=1)
    WHERE exercici = var_exercici;          

    -- percentatge de ciutadans que mai han votat en un exercici
    /* calculem els ciutadans que han pogut votar en alguna de les votacions finalitzades
       en l'any, sern ciutadans que pertanyien a un cens assignat a aquestes votacions i que 
       abans de la data de finalitzacio del periode de votacio eran majors d'edat*/
    SELECT count( distinct ciutada.idciutada) INTO var_numCiutadans 
    FROM ciutada
      INNER JOIN padrocens ON ciutada.idciutada = padrocens.idciutada
      INNER JOIN censvotacio ON padrocens.idcens = censvotacio.idcens
      INNER JOIN votacio ON censvotacio.idvotacio = votacio.idvotacio
    WHERE extract(year from votacio.datafinal)=var_exercici AND votacio.estat='F'
          AND datanaixement <= add_months(votacio.datafinal, -(var_MajoriaEdat *12));

    /* calculem els ciutadans que han votat votacions finalitzades en l'any*/
    SELECT count( distinct idciutada) INTO var_numCiutadansVotat 
    FROM participacio
      INNER JOIN votacio ON participacio.idvotacio=votacio.idvotacio
    WHERE extract(year from datafinal)=var_exercici AND estat='F';

    -- calculem el percentatge de ciutadans que mai han votat, havent-ho pogut fer
    UPDATE stat_any 
      SET percenciutnovotat=ROUND(((var_numCiutadans-var_numCiutadansVotat)/var_numCiutadans)*100,2)
      WHERE exercici=var_exercici;
    
    -- recalcula valors (stat_cens_any) (idCens)
    OPEN c_cens_votacio;
    LOOP
      	FETCH c_cens_votacio
	INTO var_idcens;
	EXIT WHEN c_cens_votacio%notfound;

        var_numStat:= 0;
        SELECT count(*) INTO var_numStat 
          FROM stat_cens_any
          WHERE idcens= var_idcens AND exercici=var_exercici;
          
        -- acumulem votaci finalitzada al cens
        IF var_numStat > 0 THEN
          UPDATE stat_cens_any SET nvotacions = nvotacions+1 WHERE idcens=var_idcens AND exercici=var_exercici;
        ELSE
          INSERT INTO stat_cens_any(idcens, exercici, nvotacions) VALUES ( var_idcens, var_exercici, 1);
        END IF;

        -- calculem les votacions finalitzades aquest any del cens, i la suma de participacions
        SELECT COUNT(*), SUM(PERCENPARTICIPACIO) INTO var_numVotacions, var_sumaParticipacio
        FROM votacio
          INNER JOIN censvotacio ON votacio.idvotacio=censvotacio.idvotacio
        WHERE censvotacio.idCens=var_idCens and votacio.estat='F' and extract(year from votacio.datafinal)=var_exercici;
        
        -- calculem la mitjana de participacio de les votacions finalitzades aquest any del cens
        UPDATE stat_cens_any 
          SET valormitjapart=ROUND((var_sumaParticipacio/var_numVotacions),2)
          WHERE idcens=var_idcens AND exercici=var_exercici;

    END LOOP;
    CLOSE c_cens_votacio;             

    -- recalcula valors (stat_pais_any) (idPais)
    OPEN c_Paisos_votacio;
    LOOP
      	FETCH c_Paisos_votacio
	INTO var_idPais;
	EXIT WHEN c_Paisos_votacio%notfound;

        var_numStat:= 0;
        SELECT count(*) INTO var_numStat 
          FROM stat_pais_any
          WHERE idpais= var_idPais AND exercici=var_exercici;
          
        IF var_numStat > 0 THEN
          UPDATE stat_pais_any SET nvotacionsfi = nvotacionsfi+1 WHERE idpais=var_idpais AND exercici=var_exercici;
        ELSE
          INSERT INTO stat_pais_any(idpais, exercici, nvotacionsfi) VALUES ( var_idpais, var_exercici, 1);
        END IF;

    END LOOP;
    CLOSE c_paisos_votacio;             
    
    -- recalcula valors (stat_votacio) 
    var_numStat:= 0;
    SELECT count(*) INTO var_numStat 
      FROM stat_votacio;
      
    IF var_numStat = 0 THEN
      INSERT INTO stat_votacio(idvotaciomespart) VALUES ( (SELECT idVotacio FROM 
                                                          (SELECT idVotacio
                                                              FROM votacio
                                                              ORDER BY percenparticipacio DESC)
                                                            WHERE ROWNUM=1));
    END IF;
    
    /*  es guarda la votaci que historicament ha tingut ms participaci
        la que historicament ha tingut ms diferencia percentual entre l'opci ms votada i la menys
        la que historicament ha tingut menys diferencia percentual entre l'opci ms votada i la menys */
    UPDATE stat_votacio 
      SET idvotaciomespart = ( SELECT idVotacio FROM 
                                (SELECT idVotacio
                                    FROM votacio
                                    ORDER BY percenparticipacio DESC)
                                  WHERE ROWNUM=1),
          idvotaciomesdifer = ( SELECT * FROM
                                  (SELECT vot.idVotacio  
                                    FROM votacio vot
                                    WHERE   ((SELECT MAX(percenvots) FROM opcionsvotacio opcvot WHERE opcvot.idvotacio=vot.idVotacio) -
                                            (SELECT MIN(percenvots) FROM opcionsvotacio opcvot WHERE opcvot.idvotacio=vot.idVotacio)) IS NOT NULL
                                            AND vot.estat='F'
                                    ORDER BY  ((SELECT MAX(percenvots) FROM opcionsvotacio opcvot WHERE opcvot.idvotacio=vot.idVotacio) -
                                              (SELECT MIN(percenvots) FROM opcionsvotacio opcvot WHERE opcvot.idvotacio=vot.idVotacio)) DESC)
                                WHERE ROWNUM=1),
          idvotaciomenysdifer = ( SELECT * FROM
                                  (SELECT vot.idVotacio  
                                    FROM votacio vot
                                    WHERE   ((SELECT MAX(percenvots) FROM opcionsvotacio opcvot WHERE opcvot.idvotacio=vot.idVotacio) -
                                            (SELECT MIN(percenvots) FROM opcionsvotacio opcvot WHERE opcvot.idvotacio=vot.idVotacio)) IS NOT NULL
                                            AND vot.estat='F'
                                    ORDER BY  ((SELECT MAX(percenvots) FROM opcionsvotacio opcvot WHERE opcvot.idvotacio=vot.idVotacio) -
                                              (SELECT MIN(percenvots) FROM opcionsvotacio opcvot WHERE opcvot.idvotacio=vot.idVotacio)) ASC)
                                WHERE ROWNUM=1),
          nmaxvotacionspersona = ( SELECT numvotacions FROM
                                    (SELECT  count(*) as numvotacions, ciutada.idpais as pais
                                      FROM participacio          
                                        INNER JOIN votacio ON participacio.idvotacio=votacio.idvotacio
                                        INNER JOIN ciutada ON participacio.idciutada=ciutada.idciutada
                                      WHERE votacio.estat = 'F'  
                                      GROUP BY participacio.idciutada, ciutada.idpais
                                      ORDER BY COUNT(*) DESC)
                                  WHERE ROWNUM=1),
          paispersmaxvots = ( SELECT pais FROM
                                    (SELECT  count(*) as numvotacions, ciutada.idpais as pais
                                      FROM participacio          
                                        INNER JOIN votacio ON participacio.idvotacio=votacio.idvotacio
                                        INNER JOIN ciutada ON participacio.idciutada=ciutada.idciutada
                                      WHERE votacio.estat = 'F'  
                                      GROUP BY participacio.idciutada, ciutada.idpais
                                      ORDER BY COUNT(*) DESC)
                                  WHERE ROWNUM=1);
    
    -- accio correcte
    RSP := 'OK';
    INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
    COMMIT;
-- control d'excepcions
EXCEPTION
       	-- problemes amb els parametres d'entrada
	WHEN e_parmEntrada THEN
		ROLLBACK;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
       	-- altres errors
	WHEN OTHERS THEN
		ROLLBACK;
                RSP := 'ERROR En la finalitzaci de la votaci: '||SQLERRM;
                INSERT INTO log(procediment, paramentrada, paramsortida) VALUES ( var_procediment, var_par_entrada, RSP);
		COMMIT;
END p_final_votacio;
/
